#open "genlex";;
#open "printf";;

let keywords =
  ["'"; "("; ")"; ","; "."; ";"; "["; "]"; "_"; "{"; "}"]
;;

let lexer = make_lexer keywords;;

let current = ref "";;

let rec parse_bibfile s =
  match s
  with [< 'Ident "%###" >] -> parse_header s
     | [< '_ >] -> parse_bibfile s
     | [< >] -> []
and parse_header s =
  match s
  with [< 'Ident "@"; parse_entry e >] -> e @ parse_bibfile s
     | [< '_ >] -> parse_header s
and parse_entry s =
  match s
  with [< 'Ident "string"; 'Kwd "{"; parse_fields _; 'Kwd "}" >] -> []
     | [< 'Ident "preamble"; 'Kwd "{"; parse_contents _; 'Kwd "}" >] -> []
     | [< 'Ident t; 'Kwd "{"; parse_key key; parse_fields ff; 'Kwd "}" >]
       -> current := key; [(("-type-",t)::("-key-",key)::ff)]
and parse_key s =
  match s
  with [< 'Kwd "." >] -> "." ^ parse_key s
     | [< parse_contents i >] -> i ^ parse_key s
     | [< >] -> ""
and parse_fields s =
  match s
  with [< 'Kwd "," >] -> parse_fields s
     | [< 'Ident label; 'Ident "="; parse_contents c >]
       -> (label, c) :: parse_fields s
     | [< >] -> []
and parse_contents s =
  match s
  with [< 'String s >] -> "\"" ^ s ^ "\""
     | [< 'Int i >] -> string_of_int i
     | [< 'Ident s >] -> s
;;

let rec print_bibfile = function
    [] -> ()
  | e::f -> print_entry e; print_bibfile f
and print_entry = function
    ("-type-", t)::("-key-", k)::fields 
    -> printf "@%s{%s,\n" t k; print_fields fields
  | _ -> ()
and print_fields = function
    [] -> printf "}\n"; ()
  | (t,c)::f -> printf "  %s = %s,\n" t c; print_fields f
;;

let get_field entry field =
  try assoc field entry
  with Not_found -> ""
;;

let pad s =
  match string_length s
  with 0 -> "  "
     | 1 -> " " ^ s
     | _ -> s
;;

let compute_key e =
   (get_field e "-type-") ^ "/" ^ (get_field e "journal") ^ "/"
   ^ (get_field e "booktitle") ^ "/" ^ (get_field e "institution") ^ "/"
   ^ (get_field e "year") ^ "/" ^ (pad (get_field e "number")) ^ "/"
   ^ (get_field e "pages")
;;

let compare_entries e1 e2 =
  le_string (compute_key e1) (compute_key e2)
;;

let sort_bibfile f = sort__sort compare_entries f;;

let main () =
  try fprintf stderr "parsing...\n"; flush stderr;
      let file = parse_bibfile (lexer (stream_of_channel stdin)) in
      fprintf stderr "sorting...\n"; flush stderr;
      let file = sort_bibfile file in
      fprintf stderr "printing...\n"; flush stderr;
      print_bibfile file
  with x -> fprintf stderr "(last good entry was %s)\n" !current;
            flush stderr;
            raise x
;;

printexc__f main ();;

flush stdout;;
flush stderr;;
